@@ -30,7 +30,7 @@
 
 #include "cogl-gpu-info-private.h"
 #include "cogl-context-private.h"
-#include "cogl-util.h"
+#include "cogl-version.h"
 
 typedef struct
 {
@@ -44,7 +44,7 @@ typedef struct CoglGpuInfoArchitectureDescription
   CoglGpuInfoArchitecture architecture;
   const char *name;
   CoglGpuInfoArchitectureFlag flags;
-  gboolean (* check_function) (const CoglGpuInfoStrings *strings);
+  CoglBool (* check_function) (const CoglGpuInfoStrings *strings);
 
 } CoglGpuInfoArchitectureDescription;
 
@@ -52,7 +52,7 @@ typedef struct
 {
   CoglGpuInfoVendor vendor;
   const char *name;
-  gboolean (* check_function) (const CoglGpuInfoStrings *strings);
+  CoglBool (* check_function) (const CoglGpuInfoStrings *strings);
   const CoglGpuInfoArchitectureDescription *architectures;
 
 } CoglGpuInfoVendorDescription;
@@ -61,18 +61,18 @@ typedef struct
 {
   CoglGpuInfoDriverPackage driver_package;
   const char *name;
-  gboolean (* check_function) (const CoglGpuInfoStrings *strings,
+  CoglBool (* check_function) (const CoglGpuInfoStrings *strings,
                                int *version_out);
 } CoglGpuInfoDriverPackageDescription;
 
-static gboolean
+static CoglBool
 _cogl_gpu_info_parse_version_string (const char *version_string,
                                      int n_components,
                                      const char **tail,
                                      int *version_ret)
 {
   int version = 0;
-  guint64 part;
+  uint64_t part;
   int i;
 
   for (i = 0; ; i++)
@@ -104,7 +104,7 @@ _cogl_gpu_info_parse_version_string (const char *version_string,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 match_phrase (const char *string, const char *phrase)
 {
   const char *part = strstr (string, phrase);
@@ -127,13 +127,13 @@ match_phrase (const char *string, const char *phrase)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_intel_vendor (const CoglGpuInfoStrings *strings)
 {
   return match_phrase (strings->renderer_string, "Intel(R)");
 }
 
-static gboolean
+static CoglBool
 check_imagination_technologies_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "Imagination Technologies") != 0)
@@ -141,7 +141,7 @@ check_imagination_technologies_vendor (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_arm_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "ARM") != 0)
@@ -149,7 +149,7 @@ check_arm_vendor (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_qualcomm_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "Qualcomm") != 0)
@@ -157,7 +157,7 @@ check_qualcomm_vendor (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_nvidia_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "NVIDIA") != 0)
@@ -166,7 +166,7 @@ check_nvidia_vendor (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_ati_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "ATI") != 0)
@@ -175,7 +175,7 @@ check_ati_vendor (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_mesa_vendor (const CoglGpuInfoStrings *strings)
 {
   if (strcmp (strings->vendor_string, "Tungsten Graphics, Inc") == 0)
@@ -186,38 +186,38 @@ check_mesa_vendor (const CoglGpuInfoStrings *strings)
   return FALSE;
 }
 
-static gboolean
+static CoglBool
 check_true (const CoglGpuInfoStrings *strings)
 {
   /* This is a last resort so it always matches */
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_sandybridge_architecture (const CoglGpuInfoStrings *strings)
 {
   return match_phrase (strings->renderer_string, "Sandybridge");
 }
 
-static gboolean
+static CoglBool
 check_llvmpipe_architecture (const CoglGpuInfoStrings *strings)
 {
   return match_phrase (strings->renderer_string, "llvmpipe");
 }
 
-static gboolean
+static CoglBool
 check_softpipe_architecture (const CoglGpuInfoStrings *strings)
 {
   return match_phrase (strings->renderer_string, "softpipe");
 }
 
-static gboolean
+static CoglBool
 check_swrast_architecture (const CoglGpuInfoStrings *strings)
 {
   return match_phrase (strings->renderer_string, "software rasterizer");
 }
 
-static gboolean
+static CoglBool
 check_sgx_architecture (const CoglGpuInfoStrings *strings)
 {
   if (strncmp (strings->renderer_string, "PowerVR SGX", 12) != 0)
@@ -226,7 +226,7 @@ check_sgx_architecture (const CoglGpuInfoStrings *strings)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_mali_architecture (const CoglGpuInfoStrings *strings)
 {
   if (strncmp (strings->renderer_string, "Mali-", 5) != 0)
@@ -397,11 +397,11 @@ _cogl_gpu_info_vendors[] =
     }
   };
 
-static gboolean
+static CoglBool
 check_mesa_driver_package (const CoglGpuInfoStrings *strings,
                            int *version_ret)
 {
-  guint64 micro_part;
+  uint64_t micro_part;
   const char *v;
 
   /* The version string should always begin a two-part GL version
@@ -448,7 +448,7 @@ check_mesa_driver_package (const CoglGpuInfoStrings *strings,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 check_unknown_driver_package (const CoglGpuInfoStrings *strings,
                               int *version_out)
 {
@@ -482,7 +482,7 @@ _cogl_gpu_info_init (CoglContext *ctx,
   int i;
 
   strings.renderer_string = (const char *) ctx->glGetString (GL_RENDERER);
-  strings.version_string = (const char *) ctx->glGetString (GL_VERSION);
+  strings.version_string = _cogl_context_get_gl_version (ctx);
   strings.vendor_string = (const char *) ctx->glGetString (GL_VENDOR);
 
   /* Determine the driver package */
