@@ -46,12 +46,25 @@ struct _CoglSwapBuffersNotifyEntry
   unsigned int id;
 };
 
+typedef struct _CoglResizeNotifyEntry CoglResizeNotifyEntry;
+
+COGL_TAILQ_HEAD (CoglResizeNotifyList, CoglResizeNotifyEntry);
+
+struct _CoglResizeNotifyEntry
+{
+  COGL_TAILQ_ENTRY (CoglResizeNotifyEntry) list_node;
+
+  CoglOnscreenResizeCallback callback;
+  void *user_data;
+  unsigned int id;
+};
+
 struct _CoglOnscreen
 {
   CoglFramebuffer  _parent;
 
 #ifdef COGL_HAS_X11_SUPPORT
-  guint32 foreign_xid;
+  uint32_t foreign_xid;
   CoglOnscreenX11MaskCallback foreign_update_mask_callback;
   void *foreign_update_mask_data;
 #endif
@@ -60,10 +73,13 @@ struct _CoglOnscreen
   HWND foreign_hwnd;
 #endif
 
-  gboolean swap_throttled;
+  CoglBool swap_throttled;
 
   CoglSwapBuffersNotifyList swap_callbacks;
 
+  CoglBool resizable;
+  CoglResizeNotifyList resize_callbacks;
+
   void *winsys;
 };
 
@@ -77,4 +93,7 @@ _cogl_framebuffer_winsys_update_size (CoglFramebuffer *framebuffer,
 void
 _cogl_onscreen_notify_swap_buffers (CoglOnscreen *onscreen);
 
+void
+_cogl_onscreen_notify_resize (CoglOnscreen *onscreen);
+
 #endif /* __COGL_ONSCREEN_PRIVATE_H */
