@@ -39,6 +39,7 @@
 #include "cogl-boxed-value.h"
 #include "cogl-pipeline-snippet-private.h"
 #include "cogl-pipeline-state.h"
+#include "cogl-framebuffer.h"
 
 #include <glib.h>
 
@@ -348,7 +349,7 @@ typedef struct
 
 typedef struct
 {
-  gboolean        enabled;
+  CoglBool        enabled;
   CoglColor       color;
   CoglFogMode     mode;
   float           density;
@@ -397,12 +398,6 @@ typedef struct
   CoglPipelineSnippetList fragment_snippets;
 } CoglPipelineBigState;
 
-typedef enum
-{
-  COGL_PIPELINE_FLAG_DIRTY_LAYERS_CACHE     = 1L<<0,
-  COGL_PIPELINE_FLAG_DIRTY_GET_LAYERS_LIST  = 1L<<1
-} CoglPipelineFlag;
-
 typedef struct
 {
   CoglPipeline *owner;
@@ -555,15 +550,15 @@ struct _CoglPipeline
 
 typedef struct _CoglPipelineFragend
 {
-  gboolean (*start) (CoglPipeline *pipeline,
+  CoglBool (*start) (CoglPipeline *pipeline,
                      int n_layers,
                      unsigned long pipelines_difference,
                      int n_tex_coord_attribs);
-  gboolean (*add_layer) (CoglPipeline *pipeline,
+  CoglBool (*add_layer) (CoglPipeline *pipeline,
                          CoglPipelineLayer *layer,
                          unsigned long layers_difference);
-  gboolean (*passthrough) (CoglPipeline *pipeline);
-  gboolean (*end) (CoglPipeline *pipeline,
+  CoglBool (*passthrough) (CoglPipeline *pipeline);
+  CoglBool (*end) (CoglPipeline *pipeline,
                    unsigned long pipelines_difference);
 
   void (*pipeline_pre_change_notify) (CoglPipeline *pipeline,
@@ -577,14 +572,15 @@ typedef struct _CoglPipelineFragend
 
 typedef struct _CoglPipelineVertend
 {
-  gboolean (*start) (CoglPipeline *pipeline,
+  CoglBool (*start) (CoglPipeline *pipeline,
                      int n_layers,
                      unsigned long pipelines_difference,
                      int n_tex_coord_attribs);
-  gboolean (*add_layer) (CoglPipeline *pipeline,
+  CoglBool (*add_layer) (CoglPipeline *pipeline,
                          CoglPipelineLayer *layer,
-                         unsigned long layers_difference);
-  gboolean (*end) (CoglPipeline *pipeline,
+                         unsigned long layers_difference,
+                         CoglFramebuffer *framebuffer);
+  CoglBool (*end) (CoglPipeline *pipeline,
                    unsigned long pipelines_difference);
 
   void (*pipeline_pre_change_notify) (CoglPipeline *pipeline,
@@ -609,7 +605,7 @@ typedef struct
   /* This is called after all of the other functions whenever the
      pipeline is flushed, even if the pipeline hasn't changed since
      the last flush */
-  void (* pre_paint) (CoglPipeline *pipeline);
+  void (* pre_paint) (CoglPipeline *pipeline, CoglFramebuffer *framebuffer);
 } CoglPipelineProgend;
 
 typedef enum
@@ -646,7 +642,7 @@ _cogl_pipeline_get_authority (CoglPipeline *pipeline,
   return authority;
 }
 
-typedef gboolean (*CoglPipelineStateComparitor) (CoglPipeline *authority0,
+typedef CoglBool (*CoglPipelineStateComparitor) (CoglPipeline *authority0,
                                                  CoglPipeline *authority1);
 
 void
@@ -659,7 +655,7 @@ void
 _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
                                   CoglPipelineState change,
                                   const CoglColor  *new_color,
-                                  gboolean          from_layer_change);
+                                  CoglBool          from_layer_change);
 
 void
 _cogl_pipeline_prune_redundant_ancestry (CoglPipeline *pipeline);
@@ -680,7 +676,7 @@ _cogl_pipeline_get_layer_with_flags (CoglPipeline *pipeline,
 #define _cogl_pipeline_get_layer(p, l) \
   _cogl_pipeline_get_layer_with_flags (p, l, 0)
 
-gboolean
+CoglBool
 _cogl_is_pipeline_layer (void *object);
 
 void
@@ -698,7 +694,7 @@ _cogl_pipeline_prune_empty_layer_difference (CoglPipeline *layers_authority,
  * able to fill your geometry according to a given Cogl pipeline.
  */
 
-gboolean
+CoglBool
 _cogl_pipeline_get_real_blend_enabled (CoglPipeline *pipeline);
 
 /*
@@ -713,11 +709,11 @@ _cogl_pipeline_pre_paint_for_layer (CoglPipeline *pipeline,
 /*
  * CoglPipelineFlushFlag:
  * @COGL_PIPELINE_FLUSH_FALLBACK_MASK: The fallback_layers member is set to
- *      a guint32 mask of the layers that can't be supported with the user
+ *      a uint32_t mask of the layers that can't be supported with the user
  *      supplied texture and need to be replaced with fallback textures. (1 =
  *      fallback, and the least significant bit = layer 0)
  * @COGL_PIPELINE_FLUSH_DISABLE_MASK: The disable_layers member is set to
- *      a guint32 mask of the layers that you want to completly disable
+ *      a uint32_t mask of the layers that you want to completly disable
  *      texturing for (1 = fallback, and the least significant bit = layer 0)
  * @COGL_PIPELINE_FLUSH_LAYER0_OVERRIDE: The layer0_override_texture member is
  *      set to a GLuint OpenGL texture name to override the texture used for
@@ -742,11 +738,11 @@ typedef enum _CoglPipelineFlushFlag
  */
 typedef struct _CoglPipelineFlushOptions
 {
-  CoglPipelineFlushFlag         flags;
+  CoglPipelineFlushFlag flags;
 
-  guint32                       fallback_layers;
-  guint32                       disable_layers;
-  CoglTexture                  *layer0_override_texture;
+  uint32_t fallback_layers;
+  uint32_t disable_layers;
+  CoglTexture *layer0_override_texture;
 } CoglPipelineFlushOptions;
 
 void
@@ -892,7 +888,7 @@ _cogl_pipeline_get_parent (CoglPipeline *pipeline);
 
 void
 _cogl_pipeline_get_colorubv (CoglPipeline *pipeline,
-                             guint8       *color);
+                             uint8_t       *color);
 
 /* XXX: At some point it could be good for this to accept a mask of
  * the state groups we are interested in comparing since we can
@@ -902,7 +898,7 @@ unsigned long
 _cogl_pipeline_compare_differences (CoglPipeline *pipeline0,
                                     CoglPipeline *pipeline1);
 
-gboolean
+CoglBool
 _cogl_pipeline_equal (CoglPipeline *pipeline0,
                       CoglPipeline *pipeline1,
                       unsigned long differences,
@@ -942,7 +938,7 @@ void
 _cogl_pipeline_set_blend_enabled (CoglPipeline *pipeline,
                                   CoglPipelineBlendEnable enable);
 
-gboolean
+CoglBool
 _cogl_pipeline_get_fog_enabled (CoglPipeline *pipeline);
 
 void
@@ -959,12 +955,12 @@ _cogl_pipeline_get_authority (CoglPipeline *pipeline,
 void
 _cogl_pipeline_add_layer_difference (CoglPipeline *pipeline,
                                      CoglPipelineLayer *layer,
-                                     gboolean inc_n_layers);
+                                     CoglBool inc_n_layers);
 
 void
 _cogl_pipeline_remove_layer_difference (CoglPipeline *pipeline,
                                         CoglPipelineLayer *layer,
-                                        gboolean dec_n_layers);
+                                        CoglBool dec_n_layers);
 
 CoglPipeline *
 _cogl_pipeline_find_equivalent_parent (CoglPipeline *pipeline,
@@ -987,7 +983,7 @@ _cogl_pipeline_prune_to_n_layers (CoglPipeline *pipeline, int n);
 const GList *
 _cogl_pipeline_get_layers (CoglPipeline *pipeline);
 
-typedef gboolean (*CoglPipelineInternalLayerCallback) (CoglPipelineLayer *layer,
+typedef CoglBool (*CoglPipelineInternalLayerCallback) (CoglPipelineLayer *layer,
                                                        void *user_data);
 
 void
@@ -995,7 +991,7 @@ _cogl_pipeline_foreach_layer_internal (CoglPipeline *pipeline,
                                        CoglPipelineInternalLayerCallback callback,
                                        void *user_data);
 
-gboolean
+CoglBool
 _cogl_pipeline_need_texture_combine_separate
                                     (CoglPipelineLayer *combine_authority);
 
