@@ -142,7 +142,9 @@ static void
 _cogl_renderer_free (CoglRenderer *renderer)
 {
   const CoglWinsysVtable *winsys = _cogl_renderer_get_winsys (renderer);
-  winsys->renderer_disconnect (renderer);
+
+  if (winsys)
+    winsys->renderer_disconnect (renderer);
 
 #ifndef HAVE_DIRECTLY_LINKED_GL_LIBRARY
   if (renderer->libgl_module)
@@ -201,7 +203,7 @@ cogl_xlib_renderer_get_foreign_display (CoglRenderer *renderer)
 
 void
 cogl_xlib_renderer_set_event_retrieval_enabled (CoglRenderer *renderer,
-                                                gboolean enable)
+                                                CoglBool enable)
 {
   _COGL_RETURN_IF_FAIL (cogl_is_renderer (renderer));
   /* NB: Renderers are considered immutable once connected */
@@ -211,7 +213,7 @@ cogl_xlib_renderer_set_event_retrieval_enabled (CoglRenderer *renderer,
 }
 #endif /* COGL_HAS_XLIB_SUPPORT */
 
-gboolean
+CoglBool
 cogl_renderer_check_onscreen_template (CoglRenderer *renderer,
                                        CoglOnscreenTemplate *onscreen_template,
                                        GError **error)
@@ -233,16 +235,31 @@ cogl_renderer_check_onscreen_template (CoglRenderer *renderer,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 _cogl_renderer_choose_driver (CoglRenderer *renderer,
                               GError **error)
 {
   const char *driver_name = g_getenv ("COGL_DRIVER");
   const char *libgl_name;
+  CoglBool support_gles2_constraint = FALSE;
+  GList *l;
 
   if (!driver_name)
     driver_name = _cogl_config_driver;
 
+  for (l = renderer->constraints; l; l = l->next)
+    {
+      CoglRendererConstraint constraint = GPOINTER_TO_UINT (l->data);
+      if (constraint == COGL_RENDERER_CONSTRAINT_SUPPORTS_COGL_GLES2)
+        {
+          support_gles2_constraint = TRUE;
+
+          if (!driver_name && renderer->driver_override == COGL_DRIVER_ANY)
+            renderer->driver_override = COGL_DRIVER_GLES2;
+          break;
+        }
+    }
+
 #ifdef HAVE_COGL_GL
   if (renderer->driver_override == COGL_DRIVER_GL ||
       (renderer->driver_override == COGL_DRIVER_ANY &&
@@ -282,7 +299,17 @@ _cogl_renderer_choose_driver (CoglRenderer *renderer,
                "No suitable driver found");
   return FALSE;
 
- found:
+found:
+
+  if (support_gles2_constraint &&
+      renderer->driver != COGL_DRIVER_GLES2)
+    {
+      g_set_error (error,
+                   COGL_RENDERER_ERROR,
+                   COGL_RENDERER_ERROR_BAD_CONSTRAINT,
+                   "No suitable driver found");
+      return FALSE;
+    }
 
 #ifndef HAVE_DIRECTLY_LINKED_GL_LIBRARY
 
@@ -305,11 +332,12 @@ _cogl_renderer_choose_driver (CoglRenderer *renderer,
 
 /* Final connection API */
 
-gboolean
+CoglBool
 cogl_renderer_connect (CoglRenderer *renderer, GError **error)
 {
   int i;
   GString *error_message;
+  CoglBool constraints_failed = FALSE;
 
   if (renderer->connected)
     return TRUE;
@@ -326,7 +354,7 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
       const CoglWinsysVtable *winsys = _cogl_winsys_vtable_getters[i]();
       GError *tmp_error = NULL;
       GList *l;
-      gboolean constraints_failed = FALSE;
+      CoglBool skip_due_to_constraints = FALSE;
 
       if (renderer->winsys_id_override != COGL_WINSYS_ID_ANY)
         {
@@ -348,12 +376,15 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
           CoglRendererConstraint constraint = GPOINTER_TO_UINT (l->data);
           if (!(winsys->constraints & constraint))
             {
-              constraints_failed = TRUE;
+              skip_due_to_constraints = TRUE;
               break;
             }
         }
-      if (constraints_failed)
-        continue;
+      if (skip_due_to_constraints)
+        {
+          constraints_failed |= TRUE;
+          continue;
+        }
 
       /* At least temporarily we will associate this winsys with
        * the renderer in-case ->renderer_connect calls API that
@@ -376,6 +407,14 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
 
   if (!renderer->connected)
     {
+      if (constraints_failed)
+        {
+          g_set_error (error, COGL_RENDERER_ERROR,
+                       COGL_RENDERER_ERROR_BAD_CONSTRAINT,
+                       "Failed to connected to any renderer due to constraints");
+          return FALSE;
+        }
+
       renderer->winsys_vtable = NULL;
       g_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
@@ -470,11 +509,12 @@ cogl_renderer_get_winsys_id (CoglRenderer *renderer)
 
 void *
 _cogl_renderer_get_proc_address (CoglRenderer *renderer,
-                                 const char *name)
+                                 const char *name,
+                                 CoglBool in_core)
 {
   const CoglWinsysVtable *winsys = _cogl_renderer_get_winsys (renderer);
 
-  return winsys->renderer_get_proc_address (renderer, name);
+  return winsys->renderer_get_proc_address (renderer, name, in_core);
 }
 
 int
