@@ -40,10 +40,11 @@
 #include "cogl-texture-2d-sliced-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-spans.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-opengl-private.h"
+#include "cogl-primitive-texture.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -129,13 +130,13 @@ _cogl_texture_2d_sliced_foreach_sub_texture_in_region (
                                          &data);
 }
 
-static guint8 *
+static uint8_t *
 _cogl_texture_2d_sliced_allocate_waste_buffer (CoglTexture2DSliced *tex_2ds,
                                                CoglPixelFormat format)
 {
   CoglSpan *last_x_span;
   CoglSpan *last_y_span;
-  guint8   *waste_buf = NULL;
+  uint8_t *waste_buf = NULL;
 
   /* If the texture has any waste then allocate a buffer big enough to
      fill the gaps */
@@ -163,7 +164,7 @@ static void
 _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
                                    CoglBitmap *source_bmp,
                                    CoglTexture2D *slice_tex,
-                                   guint8 *waste_buf,
+                                   uint8_t *waste_buf,
                                    CoglSpan *x_span,
                                    CoglSpan *y_span,
                                    CoglSpanIter *x_iter,
@@ -173,7 +174,7 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
                                    int dst_x,
                                    int dst_y)
 {
-  gboolean need_x, need_y;
+  CoglBool need_x, need_y;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -192,9 +193,9 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
       int bmp_rowstride = cogl_bitmap_get_rowstride (source_bmp);
       CoglPixelFormat source_format = cogl_bitmap_get_format (source_bmp);
       int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
-      guint8 *bmp_data;
-      const guint8 *src;
-      guint8 *dst;
+      uint8_t *bmp_data;
+      const uint8_t *src;
+      uint8_t *dst;
       unsigned int wy, wx;
       CoglBitmap *waste_bmp;
 
@@ -305,16 +306,16 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
     }
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_upload_to_gl (CoglTexture2DSliced *tex_2ds,
                                       CoglBitmap          *bmp)
 {
-  CoglSpan        *x_span;
-  CoglSpan        *y_span;
-  CoglTexture2D   *slice_tex;
-  int              x, y;
-  guint8          *waste_buf;
-  CoglPixelFormat  bmp_format;
+  CoglSpan *x_span;
+  CoglSpan *y_span;
+  CoglTexture2D *slice_tex;
+  int x, y;
+  uint8_t *waste_buf;
+  CoglPixelFormat bmp_format;
 
   bmp_format = cogl_bitmap_get_format (bmp);
 
@@ -381,7 +382,7 @@ _cogl_texture_2d_sliced_upload_to_gl (CoglTexture2DSliced *tex_2ds,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_upload_subregion_to_gl (CoglTexture2DSliced *tex_2ds,
                                                 int          src_x,
                                                 int          src_y,
@@ -395,14 +396,14 @@ _cogl_texture_2d_sliced_upload_subregion_to_gl (CoglTexture2DSliced *tex_2ds,
 {
   CoglSpan *x_span;
   CoglSpan *y_span;
-  CoglSpanIter      x_iter;
-  CoglSpanIter      y_iter;
-  CoglTexture2D    *slice_tex;
-  int               source_x = 0, source_y = 0;
-  int               inter_w = 0, inter_h = 0;
-  int               local_x = 0, local_y = 0;
-  guint8           *waste_buf;
-  CoglPixelFormat   source_format;
+  CoglSpanIter x_iter;
+  CoglSpanIter y_iter;
+  CoglTexture2D *slice_tex;
+  int source_x = 0, source_y = 0;
+  int inter_w = 0, inter_h = 0;
+  int local_x = 0, local_y = 0;
+  uint8_t *waste_buf;
+  CoglPixelFormat source_format;
 
   source_format = cogl_bitmap_get_format (source_bmp);
 
@@ -605,7 +606,7 @@ _cogl_texture_2d_sliced_set_wrap_mode_parameters (CoglTexture *tex,
     }
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_slices_create (CoglContext *ctx,
                                        CoglTexture2DSliced *tex_2ds,
                                        int width, int height,
@@ -621,6 +622,7 @@ _cogl_texture_2d_sliced_slices_create (CoglContext *ctx,
   CoglSpan *x_span;
   CoglSpan *y_span;
   GLenum gl_intformat;
+  GLenum gl_format;
   GLenum gl_type;
 
   int   (*slices_for_size) (int, int, int, GArray*);
@@ -639,10 +641,11 @@ _cogl_texture_2d_sliced_slices_create (CoglContext *ctx,
       slices_for_size = _cogl_pot_slices_for_size;
     }
 
-  ctx->texture_driver->pixel_format_to_gl (format,
-                                           &gl_intformat,
-                                           NULL,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          format,
+                                          &gl_intformat,
+                                          &gl_format,
+                                          &gl_type);
 
   /* Negative number means no slicing forced by the user */
   if (tex_2ds->max_waste <= -1)
@@ -650,8 +653,10 @@ _cogl_texture_2d_sliced_slices_create (CoglContext *ctx,
       CoglSpan span;
 
       /* Check if size supported else bail out */
-      if (!ctx->texture_driver->size_supported (GL_TEXTURE_2D,
+      if (!ctx->texture_driver->size_supported (ctx,
+                                                GL_TEXTURE_2D,
                                                 gl_intformat,
+                                                gl_format,
                                                 gl_type,
                                                 max_width,
                                                 max_height))
@@ -684,8 +689,10 @@ _cogl_texture_2d_sliced_slices_create (CoglContext *ctx,
   else
     {
       /* Decrease the size of largest slice until supported by GL */
-      while (!ctx->texture_driver->size_supported (GL_TEXTURE_2D,
+      while (!ctx->texture_driver->size_supported (ctx,
+                                                   GL_TEXTURE_2D,
                                                    gl_intformat,
+                                                   gl_format,
                                                    gl_type,
                                                    max_width,
                                                    max_height))
@@ -786,7 +793,7 @@ _cogl_texture_2d_sliced_slices_free (CoglTexture2DSliced *tex_2ds)
         {
           CoglTexture2D *slice_tex =
             g_array_index (tex_2ds->slice_textures, CoglTexture2D *, i);
-          cogl_handle_unref (slice_tex);
+          cogl_object_unref (slice_tex);
         }
 
       g_array_free (tex_2ds->slice_textures, TRUE);
@@ -802,7 +809,7 @@ _cogl_texture_2d_sliced_free (CoglTexture2DSliced *tex_2ds)
   _cogl_texture_free (COGL_TEXTURE (tex_2ds));
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_init_base (CoglContext *ctx,
                                    CoglTexture2DSliced *tex_2ds,
                                    int width,
@@ -863,7 +870,7 @@ cogl_texture_2d_sliced_new_with_size (CoglContext *ctx,
       return NULL;
     }
 
-  return _cogl_texture_2d_sliced_handle_new (tex_2ds);
+  return _cogl_texture_2d_sliced_object_new (tex_2ds);
 }
 
 CoglTexture2DSliced *
@@ -877,11 +884,13 @@ _cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap      *bmp,
   GLenum               gl_format;
   GLenum               gl_type;
   int                  width, height;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
+  CoglContext         *ctx;
+  int                  i;
 
   _COGL_RETURN_VAL_IF_FAIL (cogl_is_bitmap (bmp), NULL);
 
+  ctx = _cogl_bitmap_get_context (bmp);
+
   width = cogl_bitmap_get_width (bmp);
   height = cogl_bitmap_get_height (bmp);
 
@@ -917,7 +926,19 @@ _cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap      *bmp,
 
   cogl_object_unref (dst_bmp);
 
-  return _cogl_texture_2d_sliced_handle_new (tex_2ds);
+  if ((flags & COGL_TEXTURE_NO_AUTO_MIPMAP))
+    for (i = 0; i < tex_2ds->slice_textures->len; i++)
+      {
+        CoglPrimitiveTexture *slice_tex;
+
+        slice_tex = g_array_index (tex_2ds->slice_textures,
+                                   CoglPrimitiveTexture *,
+                                   i);
+
+        cogl_primitive_texture_set_auto_mipmap (slice_tex, FALSE);
+      }
+
+  return _cogl_texture_2d_sliced_object_new (tex_2ds);
 
  error:
   cogl_object_unref (dst_bmp);
@@ -1012,10 +1033,10 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
 
   g_array_append_val (tex_2ds->slice_textures, tex_2d);
 
-  return _cogl_texture_2d_sliced_handle_new (tex_2ds);
+  return _cogl_texture_2d_sliced_object_new (tex_2ds);
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_is_foreign (CoglTexture *tex)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
@@ -1038,7 +1059,7 @@ _cogl_texture_2d_sliced_get_max_waste (CoglTexture *tex)
   return tex_2ds->max_waste;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_is_sliced (CoglTexture *tex)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
@@ -1052,7 +1073,7 @@ _cogl_texture_2d_sliced_is_sliced (CoglTexture *tex)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_can_hardware_repeat (CoglTexture *tex)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
@@ -1103,7 +1124,7 @@ static CoglTransformResult
 _cogl_texture_2d_sliced_transform_quad_coords_to_gl (CoglTexture *tex,
                                                      float *coords)
 {
-  gboolean need_repeat = FALSE;
+  CoglBool need_repeat = FALSE;
   int i;
 
   /* This is a bit lazy - in the case where the quad lies entirely
@@ -1128,7 +1149,7 @@ _cogl_texture_2d_sliced_transform_quad_coords_to_gl (CoglTexture *tex,
           ? COGL_TRANSFORM_HARDWARE_REPEAT : COGL_TRANSFORM_NO_REPEAT);
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_get_gl_texture (CoglTexture *tex,
                                         GLuint *out_gl_handle,
                                         GLenum *out_gl_target)
@@ -1211,7 +1232,7 @@ _cogl_texture_2d_sliced_ensure_non_quad_rendering (CoglTexture *tex)
     }
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_sliced_set_region (CoglTexture    *tex,
                                     int             src_x,
                                     int             src_y,
@@ -1299,6 +1320,7 @@ _cogl_texture_2d_sliced_get_type (CoglTexture *tex)
 static const CoglTextureVtable
 cogl_texture_2d_sliced_vtable =
   {
+    FALSE, /* not primitive */
     _cogl_texture_2d_sliced_set_region,
     NULL, /* get_data */
     _cogl_texture_2d_sliced_foreach_sub_texture_in_region,
@@ -1317,5 +1339,6 @@ cogl_texture_2d_sliced_vtable =
     _cogl_texture_2d_sliced_get_width,
     _cogl_texture_2d_sliced_get_height,
     _cogl_texture_2d_sliced_get_type,
-    _cogl_texture_2d_sliced_is_foreign
+    _cogl_texture_2d_sliced_is_foreign,
+    NULL /* set_auto_mipmap */
   };
