@@ -34,7 +34,7 @@
 #include "cogl-texture-2d-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-framebuffer-private.h"
@@ -103,31 +103,35 @@ _cogl_texture_2d_free (CoglTexture2D *tex_2d)
   _cogl_texture_free (COGL_TEXTURE (tex_2d));
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_can_create (unsigned int width,
                              unsigned int height,
                              CoglPixelFormat internal_format)
 {
   GLenum gl_intformat;
+  GLenum gl_format;
   GLenum gl_type;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
   /* If NPOT textures aren't supported then the size must be a power
      of two */
-  if (!cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT) &&
+  if (!cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_BASIC) &&
       (!_cogl_util_is_pot (width) ||
        !_cogl_util_is_pot (height)))
     return FALSE;
 
-  ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                           &gl_intformat,
-                                           NULL,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          internal_format,
+                                          &gl_intformat,
+                                          &gl_format,
+                                          &gl_type);
 
   /* Check that the driver can create a texture with that size */
-  if (!ctx->texture_driver->size_supported (GL_TEXTURE_2D,
+  if (!ctx->texture_driver->size_supported (ctx,
+                                            GL_TEXTURE_2D,
                                             gl_intformat,
+                                            gl_format,
                                             gl_type,
                                             width,
                                             height))
@@ -136,10 +140,18 @@ _cogl_texture_2d_can_create (unsigned int width,
   return TRUE;
 }
 
+static void
+_cogl_texture_2d_set_auto_mipmap (CoglTexture *tex,
+                                  CoglBool value)
+{
+  CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
+
+  tex_2d->auto_mipmap = value;
+}
+
 static CoglTexture2D *
 _cogl_texture_2d_create_base (unsigned int     width,
                               unsigned int     height,
-                              CoglTextureFlags flags,
                               CoglPixelFormat  internal_format)
 {
   CoglTexture2D *tex_2d = g_new (CoglTexture2D, 1);
@@ -150,7 +162,7 @@ _cogl_texture_2d_create_base (unsigned int     width,
   tex_2d->width = width;
   tex_2d->height = height;
   tex_2d->mipmaps_dirty = TRUE;
-  tex_2d->auto_mipmap = (flags & COGL_TEXTURE_NO_AUTO_MIPMAP) == 0;
+  tex_2d->auto_mipmap = TRUE;
 
   /* We default to GL_LINEAR for both filters */
   tex_2d->min_filter = GL_LINEAR;
@@ -192,40 +204,41 @@ cogl_texture_2d_new_with_size (CoglContext *ctx,
       return NULL;
     }
 
-  internal_format = ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                                             &gl_intformat,
-                                                             &gl_format,
-                                                             &gl_type);
+  internal_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                                            internal_format,
+                                                            &gl_intformat,
+                                                            &gl_format,
+                                                            &gl_type);
 
-  tex_2d = _cogl_texture_2d_create_base (width, height, COGL_TEXTURE_NONE,
+  tex_2d = _cogl_texture_2d_create_base (width, height,
                                          internal_format);
 
-  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->gen (ctx, GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
   GE( ctx, glTexImage2D (GL_TEXTURE_2D, 0, gl_intformat,
                          width, height, 0, gl_format, gl_type, NULL) );
 
-  return _cogl_texture_2d_handle_new (tex_2d);
+  return _cogl_texture_2d_object_new (tex_2d);
 }
 
-CoglHandle
-_cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
-                                  CoglTextureFlags flags,
-                                  CoglPixelFormat  internal_format,
-                                  GError         **error)
+CoglTexture2D *
+cogl_texture_2d_new_from_bitmap (CoglBitmap *bmp,
+                                 CoglPixelFormat internal_format,
+                                 GError **error)
 {
   CoglTexture2D *tex_2d;
-  CoglBitmap    *dst_bmp;
-  GLenum         gl_intformat;
-  GLenum         gl_format;
-  GLenum         gl_type;
-  guint8        *data;
+  CoglBitmap *dst_bmp;
+  GLenum gl_intformat;
+  GLenum gl_format;
+  GLenum gl_type;
+  uint8_t *data;
+  CoglContext *ctx;
 
-  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+  _COGL_RETURN_VAL_IF_FAIL (bmp != NULL, NULL);
 
-  _COGL_RETURN_VAL_IF_FAIL (bmp != NULL, COGL_INVALID_HANDLE);
+  ctx = _cogl_bitmap_get_context (bmp);
 
   internal_format =
     _cogl_texture_determine_internal_format (cogl_bitmap_get_format (bmp),
@@ -258,7 +271,6 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
 
   tex_2d = _cogl_texture_2d_create_base (cogl_bitmap_get_width (bmp),
                                          cogl_bitmap_get_height (bmp),
-                                         flags,
                                          internal_format);
 
   /* Keep a copy of the first pixel so that if glGenerateMipmap isn't
@@ -276,8 +288,9 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
       _cogl_bitmap_unmap (dst_bmp);
     }
 
-  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
-  ctx->texture_driver->upload_to_gl (GL_TEXTURE_2D,
+  ctx->texture_driver->gen (ctx, GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->upload_to_gl (ctx,
+                                     GL_TEXTURE_2D,
                                      tex_2d->gl_texture,
                                      FALSE,
                                      dst_bmp,
@@ -289,7 +302,7 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
 
   cogl_object_unref (dst_bmp);
 
-  return _cogl_texture_2d_handle_new (tex_2d);
+  return _cogl_texture_2d_object_new (tex_2d);
 }
 
 CoglTexture2D *
@@ -299,11 +312,11 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
                                CoglPixelFormat format,
                                CoglPixelFormat internal_format,
                                int rowstride,
-                               const guint8 *data,
+                               const uint8_t *data,
                                GError **error)
 {
   CoglBitmap *bmp;
-  CoglHandle tex;
+  CoglTexture2D *tex_2d;
 
   _COGL_RETURN_VAL_IF_FAIL (format != COGL_PIXEL_FORMAT_ANY, NULL);
   _COGL_RETURN_VAL_IF_FAIL (data != NULL, NULL);
@@ -317,15 +330,15 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
                                   width, height,
                                   format,
                                   rowstride,
-                                  (guint8 *) data);
+                                  (uint8_t *) data);
 
-  tex =_cogl_texture_2d_new_from_bitmap (bmp, COGL_TEXTURE_NONE,
-                                         internal_format,
-                                         error);
+  tex_2d = cogl_texture_2d_new_from_bitmap (bmp,
+                                            internal_format,
+                                            error);
 
   cogl_object_unref (bmp);
 
-  return tex;
+  return tex_2d;
 }
 
 CoglTexture2D *
@@ -345,12 +358,12 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
   GLenum         gl_int_format = 0;
   CoglTexture2D *tex_2d;
 
-  if (!ctx->texture_driver->allows_foreign_gl_target (GL_TEXTURE_2D))
-    return COGL_INVALID_HANDLE;
+  if (!ctx->texture_driver->allows_foreign_gl_target (ctx, GL_TEXTURE_2D))
+    return NULL;
 
   /* Make sure it is a valid GL texture object */
   if (!ctx->glIsTexture (gl_handle))
-    return COGL_INVALID_HANDLE;
+    return NULL;
 
   /* Make sure binding succeeds */
   while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
@@ -358,7 +371,7 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D, gl_handle, TRUE);
   if (ctx->glGetError () != GL_NO_ERROR)
-    return COGL_INVALID_HANDLE;
+    return NULL;
 
   /* Obtain texture parameters
      (only level 0 we are interested in) */
@@ -382,19 +395,21 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
 
       /* If we can query GL for the actual pixel format then we'll ignore
          the passed in format and use that. */
-      if (!ctx->texture_driver->pixel_format_from_gl_internal (gl_int_format,
-                                                               &format))
-        return COGL_INVALID_HANDLE;
+      if (!ctx->driver_vtable->pixel_format_from_gl_internal (ctx,
+                                                              gl_int_format,
+                                                              &format))
+        return NULL;
     }
   else
 #endif
     {
       /* Otherwise we'll assume we can derive the GL format from the
          passed in format */
-      ctx->texture_driver->pixel_format_to_gl (format,
-                                               &gl_int_format,
-                                               NULL,
-                                               NULL);
+      ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                              format,
+                                              &gl_int_format,
+                                              NULL,
+                                              NULL);
     }
 
   /* Note: We always trust the given width and height without querying
@@ -406,11 +421,11 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
 
   /* Validate width and height */
   if (width <= 0 || height <= 0)
-    return COGL_INVALID_HANDLE;
+    return NULL;
 
   /* Compressed texture images not supported */
   if (gl_compressed == GL_TRUE)
-    return COGL_INVALID_HANDLE;
+    return NULL;
 
   /* Note: previously this code would query the texture object for
      whether it has GL_GENERATE_MIPMAP enabled to determine whether to
@@ -425,8 +440,8 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
 
   /* Create new texture */
   tex_2d = _cogl_texture_2d_create_base (width, height,
-                                         COGL_TEXTURE_NO_AUTO_MIPMAP,
                                          format);
+  _cogl_texture_2d_set_auto_mipmap (COGL_TEXTURE (tex_2d), FALSE);
 
   /* Setup bitmap info */
   tex_2d->is_foreign = TRUE;
@@ -441,7 +456,7 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
   tex_2d->min_filter = GL_FALSE;
   tex_2d->mag_filter = GL_FALSE;
 
-  return _cogl_texture_2d_handle_new (tex_2d);
+  return _cogl_texture_2d_object_new (tex_2d);
 }
 
 #if defined (COGL_HAS_EGL_SUPPORT) && defined (EGL_KHR_image_base)
@@ -467,10 +482,10 @@ _cogl_egl_texture_2d_new_from_image (CoglContext *ctx,
                         COGL_PRIVATE_FEATURE_TEXTURE_2D_FROM_EGL_IMAGE,
                         NULL);
 
-  tex_2d = _cogl_texture_2d_create_base (width, height, COGL_TEXTURE_NONE,
+  tex_2d = _cogl_texture_2d_create_base (width, height,
                                          format);
 
-  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->gen (ctx, GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    FALSE);
@@ -487,7 +502,7 @@ _cogl_egl_texture_2d_new_from_image (CoglContext *ctx,
       return NULL;
     }
 
-  return _cogl_texture_2d_handle_new (tex_2d);
+  return _cogl_texture_2d_object_new (tex_2d);
 }
 #endif /* defined (COGL_HAS_EGL_SUPPORT) && defined (EGL_KHR_image_base) */
 
@@ -561,16 +576,16 @@ cogl_wayland_texture_2d_new_from_buffer (CoglContext *ctx,
 #endif /* COGL_HAS_WAYLAND_EGL_SERVER_SUPPORT */
 
 void
-_cogl_texture_2d_externally_modified (CoglHandle handle)
+_cogl_texture_2d_externally_modified (CoglTexture *texture)
 {
-  if (!cogl_is_texture_2d (handle))
+  if (!cogl_is_texture_2d (texture))
     return;
 
-  COGL_TEXTURE_2D (handle)->mipmaps_dirty = TRUE;
+  COGL_TEXTURE_2D (texture)->mipmaps_dirty = TRUE;
 }
 
 void
-_cogl_texture_2d_copy_from_framebuffer (CoglHandle handle,
+_cogl_texture_2d_copy_from_framebuffer (CoglTexture2D *tex_2d,
                                         int dst_x,
                                         int dst_y,
                                         int src_x,
@@ -578,13 +593,9 @@ _cogl_texture_2d_copy_from_framebuffer (CoglHandle handle,
                                         int width,
                                         int height)
 {
-  CoglTexture2D *tex_2d;
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _COGL_RETURN_IF_FAIL (cogl_is_texture_2d (handle));
-
-  tex_2d = COGL_TEXTURE_2D (handle);
+  _COGL_RETURN_IF_FAIL (cogl_is_texture_2d (tex_2d));
 
   /* Make sure the current framebuffers are bound, though we don't need to
    * flush the clip state here since we aren't going to draw to the
@@ -613,16 +624,25 @@ _cogl_texture_2d_get_max_waste (CoglTexture *tex)
   return -1;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_is_sliced (CoglTexture *tex)
 {
   return FALSE;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_can_hardware_repeat (CoglTexture *tex)
 {
-  return TRUE;
+  CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
+
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  if (cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_REPEAT) ||
+      (_cogl_util_is_pot (tex_2d->width) &&
+       _cogl_util_is_pot (tex_2d->height)))
+    return TRUE;
+  else
+    return FALSE;
 }
 
 static void
@@ -641,18 +661,22 @@ _cogl_texture_2d_transform_quad_coords_to_gl (CoglTexture *tex,
   /* The texture coordinates map directly so we don't need to do
      anything other than check for repeats */
 
-  gboolean need_repeat = FALSE;
   int i;
 
   for (i = 0; i < 4; i++)
     if (coords[i] < 0.0f || coords[i] > 1.0f)
-      need_repeat = TRUE;
+      {
+        /* Repeat is needed */
+        return (_cogl_texture_2d_can_hardware_repeat (tex) ?
+                COGL_TRANSFORM_HARDWARE_REPEAT :
+                COGL_TRANSFORM_SOFTWARE_REPEAT);
+      }
 
-  return (need_repeat ? COGL_TRANSFORM_HARDWARE_REPEAT
-          : COGL_TRANSFORM_NO_REPEAT);
+  /* No repeat is needed */
+  return COGL_TRANSFORM_NO_REPEAT;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_get_gl_texture (CoglTexture *tex,
                                  GLuint *out_gl_handle,
                                  GLenum *out_gl_target)
@@ -712,7 +736,7 @@ _cogl_texture_2d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
          available we'll fallback to temporarily enabling
          GL_GENERATE_MIPMAP and reuploading the first pixel */
       if (cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
-        ctx->texture_driver->gl_generate_mipmaps (GL_TEXTURE_2D);
+        ctx->texture_driver->gl_generate_mipmaps (ctx, GL_TEXTURE_2D);
 #if defined(HAVE_COGL_GLES) || defined(HAVE_COGL_GL)
       else
         {
@@ -739,7 +763,7 @@ _cogl_texture_2d_ensure_non_quad_rendering (CoglTexture *tex)
   /* Nothing needs to be done */
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_set_region (CoglTexture    *tex,
                              int             src_x,
                              int             src_y,
@@ -750,9 +774,9 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
                              CoglBitmap     *bmp)
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
-  GLenum         gl_format;
-  GLenum         gl_type;
-  guint8        *data;
+  GLenum gl_format;
+  GLenum gl_type;
+  uint8_t *data;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
@@ -780,7 +804,8 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
     }
 
   /* Send data to GL */
-  ctx->texture_driver->upload_subregion_to_gl (GL_TEXTURE_2D,
+  ctx->texture_driver->upload_subregion_to_gl (ctx,
+                                               GL_TEXTURE_2D,
                                                tex_2d->gl_texture,
                                                FALSE,
                                                src_x, src_y,
@@ -797,11 +822,11 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   return TRUE;
 }
 
-static gboolean
-_cogl_texture_2d_get_data (CoglTexture     *tex,
-                           CoglPixelFormat  format,
-                           unsigned int     rowstride,
-                           guint8          *data)
+static CoglBool
+_cogl_texture_2d_get_data (CoglTexture *tex,
+                           CoglPixelFormat format,
+                           unsigned int rowstride,
+                           uint8_t *data)
 {
   CoglTexture2D   *tex_2d = COGL_TEXTURE_2D (tex);
   int              bpp;
@@ -812,17 +837,22 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
 
   bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
 
-  ctx->texture_driver->pixel_format_to_gl (format,
-                                           NULL, /* internal format */
-                                           &gl_format,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          format,
+                                          NULL, /* internal format */
+                                          &gl_format,
+                                          &gl_type);
 
-  ctx->texture_driver->prep_gl_for_pixels_download (rowstride, bpp);
+  ctx->texture_driver->prep_gl_for_pixels_download (ctx,
+                                                    rowstride,
+                                                    tex_2d->width,
+                                                    bpp);
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
-  return ctx->texture_driver->gl_get_tex_image (GL_TEXTURE_2D,
+  return ctx->texture_driver->gl_get_tex_image (ctx,
+                                                GL_TEXTURE_2D,
                                                 gl_format,
                                                 gl_type,
                                                 data);
@@ -852,7 +882,7 @@ _cogl_texture_2d_get_height (CoglTexture *tex)
   return COGL_TEXTURE_2D (tex)->height;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_2d_is_foreign (CoglTexture *tex)
 {
   return COGL_TEXTURE_2D (tex)->is_foreign;
@@ -867,6 +897,7 @@ _cogl_texture_2d_get_type (CoglTexture *tex)
 static const CoglTextureVtable
 cogl_texture_2d_vtable =
   {
+    TRUE, /* primitive */
     _cogl_texture_2d_set_region,
     _cogl_texture_2d_get_data,
     NULL, /* foreach_sub_texture_in_region */
@@ -885,5 +916,6 @@ cogl_texture_2d_vtable =
     _cogl_texture_2d_get_width,
     _cogl_texture_2d_get_height,
     _cogl_texture_2d_get_type,
-    _cogl_texture_2d_is_foreign
+    _cogl_texture_2d_is_foreign,
+    _cogl_texture_2d_set_auto_mipmap
   };
