@@ -47,12 +47,13 @@
 #include "cogl-atlas-texture-private.h"
 #include "cogl-pipeline.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-object-private.h"
 #include "cogl-primitives.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl1-context.h"
 #include "cogl-sub-texture.h"
+#include "cogl-primitive-texture.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -65,8 +66,8 @@ cogl_texture_error_quark (void)
 }
 
 /* XXX:
- * The CoglHandle macros don't support any form of inheritance, so for
- * now we implement the CoglHandle support for the CoglTexture
+ * The CoglObject macros don't support any form of inheritance, so for
+ * now we implement the CoglObject support for the CoglTexture
  * abstract class manually.
  */
 
@@ -78,7 +79,7 @@ _cogl_texture_register_texture_type (const CoglObjectClass *klass)
   ctxt->texture_types = g_slist_prepend (ctxt->texture_types, (void *) klass);
 }
 
-gboolean
+CoglBool
 cogl_is_texture (void *object)
 {
   CoglObject *obj = (CoglObject *)object;
@@ -139,7 +140,7 @@ _cogl_texture_free (CoglTexture *texture)
   g_free (texture);
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_needs_premult_conversion (CoglPixelFormat src_format,
                                         CoglPixelFormat dst_format)
 {
@@ -166,6 +167,15 @@ _cogl_texture_determine_internal_format (CoglPixelFormat src_format,
         return src_format;
     }
   else
+    /* XXX: It might be nice to make this match the component ordering
+       of the source format when the formats are otherwise the same
+       because on GL there is no way to specify the ordering of the
+       internal format. However when using GLES with the
+       GL_EXT_texture_format_BGRA8888 the order of the internal format
+       becomes important because it must exactly match the format of
+       the uploaded data. That means that if someone creates a texture
+       with some RGBA data and then later tries to upload BGRA data we
+       do actually have to swizzle the components */
     return dst_format;
 }
 
@@ -212,24 +222,27 @@ _cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
       /* Use the source format from the src bitmap type and the internal
          format from the dst format type so that GL can do the
          conversion */
-      ctx->texture_driver->pixel_format_to_gl (src_format,
-                                               NULL, /* internal format */
-                                               out_glformat,
-                                               out_gltype);
-      ctx->texture_driver->pixel_format_to_gl (dst_format,
-                                               out_glintformat,
-                                               NULL,
-                                               NULL);
+      ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                              src_format,
+                                              NULL, /* internal format */
+                                              out_glformat,
+                                              out_gltype);
+      ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                              dst_format,
+                                              out_glintformat,
+                                              NULL,
+                                              NULL);
 
     }
   else
     {
       CoglPixelFormat closest_format;
 
-      closest_format = ctx->texture_driver->pixel_format_to_gl (dst_format,
-                                                                out_glintformat,
-                                                                out_glformat,
-                                                                out_gltype);
+      closest_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                                               dst_format,
+                                                               out_glintformat,
+                                                               out_glformat,
+                                                               out_gltype);
 
       if (closest_format != src_format)
         dst_bmp = _cogl_bitmap_convert (src_bmp, closest_format);
@@ -243,34 +256,47 @@ _cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
   return dst_bmp;
 }
 
+static inline int
+calculate_alignment (int rowstride)
+{
+  int alignment = 1 << (_cogl_util_ffs (rowstride) - 1);
+
+  return MIN (alignment, 8);
+}
+
 void
 _cogl_texture_prep_gl_alignment_for_pixels_upload (int pixels_rowstride)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (!(pixels_rowstride & 0x7))
-    GE( ctx, glPixelStorei (GL_UNPACK_ALIGNMENT, 8) );
-  else if (!(pixels_rowstride & 0x3))
-    GE( ctx, glPixelStorei (GL_UNPACK_ALIGNMENT, 4) );
-  else if (!(pixels_rowstride & 0x1))
-    GE( ctx, glPixelStorei (GL_UNPACK_ALIGNMENT, 2) );
-  else
-    GE( ctx, glPixelStorei (GL_UNPACK_ALIGNMENT, 1) );
+  GE( ctx, glPixelStorei (GL_UNPACK_ALIGNMENT,
+                          calculate_alignment (pixels_rowstride)) );
 }
 
 void
-_cogl_texture_prep_gl_alignment_for_pixels_download (int pixels_rowstride)
+_cogl_texture_prep_gl_alignment_for_pixels_download (int bpp,
+                                                     int width,
+                                                     int rowstride)
 {
+  int alignment;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (!(pixels_rowstride & 0x7))
-    GE( ctx, glPixelStorei (GL_PACK_ALIGNMENT, 8) );
-  else if (!(pixels_rowstride & 0x3))
-    GE( ctx, glPixelStorei (GL_PACK_ALIGNMENT, 4) );
-  else if (!(pixels_rowstride & 0x1))
-    GE( ctx, glPixelStorei (GL_PACK_ALIGNMENT, 2) );
+  /* If no padding is needed then we can always use an alignment of 1.
+   * We want to do this even though it is equivalent to the alignment
+   * of the rowstride because the Intel driver in Mesa currently has
+   * an optimisation when reading data into a PBO that only works if
+   * the alignment is exactly 1.
+   *
+   * https://bugs.freedesktop.org/show_bug.cgi?id=46632
+   */
+
+  if (rowstride == bpp * width)
+    alignment = 1;
   else
-    GE( ctx, glPixelStorei (GL_PACK_ALIGNMENT, 1) );
+    alignment = calculate_alignment (rowstride);
+
+  GE( ctx, glPixelStorei (GL_PACK_ALIGNMENT, alignment) );
 }
 
 /* FIXME: wrap modes should be set on pipelines not textures */
@@ -294,17 +320,30 @@ cogl_texture_new_with_size (unsigned int     width,
 {
   CoglTexture *tex;
 
-  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+  _COGL_GET_CONTEXT (ctx, NULL);
 
-  /* First try creating a fast-path non-sliced texture */
-  tex = COGL_TEXTURE (cogl_texture_2d_new_with_size (ctx,
-                                                     width, height,
-                                                     internal_format,
-                                                     NULL));
-
-  /* If it fails resort to sliced textures */
-  if (tex == NULL)
+  if ((_cogl_util_is_pot (width) && _cogl_util_is_pot (height)) ||
+      (cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_BASIC) &&
+       cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_MIPMAP)))
     {
+      /* First try creating a fast-path non-sliced texture */
+      tex = COGL_TEXTURE (cogl_texture_2d_new_with_size (ctx,
+                                                         width, height,
+                                                         internal_format,
+                                                         NULL));
+    }
+  else
+    tex = NULL;
+
+  if (tex)
+    {
+      CoglBool auto_mipmap = !(flags & COGL_TEXTURE_NO_AUTO_MIPMAP);
+      cogl_primitive_texture_set_auto_mipmap (COGL_PRIMITIVE_TEXTURE (tex),
+                                              auto_mipmap);
+    }
+  else
+    {
+      /* If it fails resort to sliced textures */
       int max_waste = flags & COGL_TEXTURE_NO_SLICING ? -1 : COGL_TEXTURE_MAX_WASTE;
       tex = COGL_TEXTURE (cogl_texture_2d_sliced_new_with_size (ctx,
                                                                 width,
@@ -318,13 +357,13 @@ cogl_texture_new_with_size (unsigned int     width,
 }
 
 CoglTexture *
-cogl_texture_new_from_data (unsigned int      width,
-			    unsigned int      height,
-                            CoglTextureFlags  flags,
-			    CoglPixelFormat   format,
-			    CoglPixelFormat   internal_format,
-			    unsigned int      rowstride,
-			    const guint8     *data)
+cogl_texture_new_from_data (unsigned int width,
+			    unsigned int height,
+                            CoglTextureFlags flags,
+			    CoglPixelFormat format,
+			    CoglPixelFormat internal_format,
+			    unsigned int rowstride,
+			    const uint8_t *data)
 {
   CoglBitmap *bmp;
   CoglTexture *tex;
@@ -346,7 +385,7 @@ cogl_texture_new_from_data (unsigned int      width,
                                   width, height,
                                   format,
                                   rowstride,
-                                  (guint8 *) data);
+                                  (uint8_t *) data);
 
   tex = cogl_texture_new_from_bitmap (bmp, flags, internal_format);
 
@@ -360,26 +399,45 @@ cogl_texture_new_from_bitmap (CoglBitmap *bitmap,
                               CoglTextureFlags flags,
                               CoglPixelFormat  internal_format)
 {
+  CoglAtlasTexture *atlas_tex;
   CoglTexture *tex;
 
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
   /* First try putting the texture in the atlas */
-  if ((tex = _cogl_atlas_texture_new_from_bitmap (bitmap,
-                                                  flags,
-                                                  internal_format)))
-    return tex;
+  if ((atlas_tex = _cogl_atlas_texture_new_from_bitmap (bitmap,
+                                                        flags,
+                                                        internal_format)))
+    return COGL_TEXTURE (atlas_tex);
 
   /* If that doesn't work try a fast path 2D texture */
-  if ((tex = _cogl_texture_2d_new_from_bitmap (bitmap,
-                                               flags,
-                                               internal_format,
-                                               NULL)))
-    return tex;
+  if ((_cogl_util_is_pot (bitmap->width) &&
+       _cogl_util_is_pot (bitmap->height)) ||
+      (cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_BASIC) &&
+       cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_MIPMAP)))
+    {
+      tex = COGL_TEXTURE (cogl_texture_2d_new_from_bitmap (bitmap,
+                                                           internal_format,
+                                                           NULL));
+    }
+  else
+    tex = NULL;
 
-  /* Otherwise create a sliced texture */
-  return
-    COGL_TEXTURE (_cogl_texture_2d_sliced_new_from_bitmap (bitmap,
-                                                           flags,
-                                                           internal_format));
+  if (tex)
+    {
+      CoglBool auto_mipmap = !(flags & COGL_TEXTURE_NO_AUTO_MIPMAP);
+      cogl_primitive_texture_set_auto_mipmap (COGL_PRIMITIVE_TEXTURE (tex),
+                                              auto_mipmap);
+    }
+  else
+    {
+      /* Otherwise create a sliced texture */
+      tex = COGL_TEXTURE (_cogl_texture_2d_sliced_new_from_bitmap (bitmap,
+                                                             flags,
+                                                             internal_format));
+    }
+
+  return tex;
 }
 
 CoglTexture *
@@ -392,6 +450,8 @@ cogl_texture_new_from_file (const char        *filename,
   CoglTexture *texture = NULL;
   CoglPixelFormat src_format;
 
+  _COGL_GET_CONTEXT (ctx, NULL);
+
   _COGL_RETURN_VAL_IF_FAIL (error == NULL || *error == NULL, NULL);
 
   bmp = cogl_bitmap_new_from_file (filename, error);
@@ -430,7 +490,7 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
       CoglTextureRectangle *texture_rectangle;
       CoglSubTexture *sub_texture;
 
-      _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+      _COGL_GET_CONTEXT (ctx, NULL);
 
       if (x_pot_waste != 0 || y_pot_waste != 0)
         {
@@ -438,7 +498,7 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
            * the texture isn't limited to power of two sizes. */
           g_warning ("You can't create a foreign GL_TEXTURE_RECTANGLE cogl "
                      "texture with waste\n");
-          return COGL_INVALID_HANDLE;
+          return NULL;
         }
 
       texture_rectangle = _cogl_texture_rectangle_new_from_foreign (gl_handle,
@@ -466,7 +526,7 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
                                                                    format));
   else
     {
-      _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+      _COGL_GET_CONTEXT (ctx, NULL);
       return COGL_TEXTURE (cogl_texture_2d_new_from_foreign (ctx,
                                                              gl_handle,
                                                              width,
@@ -476,7 +536,7 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
     }
 }
 
-gboolean
+CoglBool
 _cogl_texture_is_foreign (CoglTexture *texture)
 {
   if (texture->vtable->is_foreign)
@@ -535,7 +595,7 @@ cogl_texture_get_max_waste (CoglTexture *texture)
   return texture->vtable->get_max_waste (texture);
 }
 
-gboolean
+CoglBool
 cogl_texture_is_sliced (CoglTexture *texture)
 {
   return texture->vtable->is_sliced (texture);
@@ -545,7 +605,7 @@ cogl_texture_is_sliced (CoglTexture *texture)
  * will be needed to iterate over multiple sub textures for regions whos
  * texture coordinates extend out of the range [0,1]
  */
-gboolean
+CoglBool
 _cogl_texture_can_hardware_repeat (CoglTexture *texture)
 {
   return texture->vtable->can_hardware_repeat (texture);
@@ -575,7 +635,7 @@ _cogl_texture_get_gl_format (CoglTexture *texture)
   return texture->vtable->get_gl_format (texture);
 }
 
-gboolean
+CoglBool
 cogl_texture_get_gl_texture (CoglTexture *texture,
 			     GLuint *out_gl_handle,
 			     GLenum *out_gl_target)
@@ -610,7 +670,7 @@ _cogl_texture_ensure_non_quad_rendering (CoglTexture *texture)
   texture->vtable->ensure_non_quad_rendering (texture);
 }
 
-gboolean
+CoglBool
 cogl_texture_set_region_from_bitmap (CoglTexture *texture,
                                      int src_x,
                                      int src_y,
@@ -620,7 +680,7 @@ cogl_texture_set_region_from_bitmap (CoglTexture *texture,
                                      unsigned int dst_height,
                                      CoglBitmap *bmp)
 {
-  gboolean ret;
+  CoglBool ret;
 
   _COGL_RETURN_VAL_IF_FAIL ((cogl_bitmap_get_width (bmp) - src_x)
                             >= dst_width, FALSE);
@@ -647,22 +707,22 @@ cogl_texture_set_region_from_bitmap (CoglTexture *texture,
   return ret;
 }
 
-gboolean
-cogl_texture_set_region (CoglTexture     *texture,
-			 int              src_x,
-			 int              src_y,
-			 int              dst_x,
-			 int              dst_y,
-			 unsigned int     dst_width,
-			 unsigned int     dst_height,
-			 int              width,
-			 int              height,
-			 CoglPixelFormat  format,
-			 unsigned int     rowstride,
-			 const guint8    *data)
+CoglBool
+cogl_texture_set_region (CoglTexture *texture,
+			 int src_x,
+			 int src_y,
+			 int dst_x,
+			 int dst_y,
+			 unsigned int dst_width,
+			 unsigned int dst_height,
+			 int width,
+			 int height,
+			 CoglPixelFormat format,
+			 unsigned int rowstride,
+			 const uint8_t *data)
 {
   CoglBitmap *source_bmp;
-  gboolean    ret;
+  CoglBool    ret;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
@@ -682,7 +742,7 @@ cogl_texture_set_region (CoglTexture     *texture,
                                          width, height,
                                          format,
                                          rowstride,
-                                         (guint8 *) data);
+                                         (uint8_t *) data);
 
   ret = cogl_texture_set_region_from_bitmap (texture,
                                              src_x, src_y,
@@ -799,7 +859,7 @@ do_texture_draw_and_read (CoglTexture *texture,
  * NB: Normally this approach isn't normally used since we can just use
  * glGetTexImage, but may be used as a fallback in some circumstances.
  */
-gboolean
+CoglBool
 _cogl_texture_draw_and_read (CoglTexture *texture,
                              CoglBitmap  *target_bmp,
                              GLuint       target_gl_format,
@@ -878,12 +938,12 @@ _cogl_texture_draw_and_read (CoglTexture *texture,
   printf ("A bits: %d\n", a_bits); */
   if ((cogl_texture_get_format (texture) & COGL_A_BIT)/* && a_bits == 0*/)
     {
-      guint8 *srcdata;
-      guint8 *dstdata;
-      guint8 *srcpixel;
-      guint8 *dstpixel;
-      int     x,y;
-      int     alpha_rowstride = bpp * target_width;
+      uint8_t *srcdata;
+      uint8_t *dstdata;
+      uint8_t *srcpixel;
+      uint8_t *dstpixel;
+      int x,y;
+      int alpha_rowstride = bpp * target_width;
 
       if ((dstdata = _cogl_bitmap_map (target_bmp,
                                        COGL_BUFFER_ACCESS_WRITE,
@@ -940,20 +1000,20 @@ _cogl_texture_draw_and_read (CoglTexture *texture,
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 get_texture_bits_via_offscreen (CoglTexture    *texture,
                                 int             x,
                                 int             y,
                                 int             width,
                                 int             height,
-                                guint8         *dst_bits,
+                                uint8_t         *dst_bits,
                                 unsigned int    dst_rowstride,
                                 CoglPixelFormat dst_format)
 {
   CoglOffscreen *offscreen;
   CoglFramebuffer *framebuffer;
   CoglBitmap *bitmap;
-  gboolean ret;
+  CoglBool ret;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
@@ -986,19 +1046,19 @@ get_texture_bits_via_offscreen (CoglTexture    *texture,
   return ret;
 }
 
-static gboolean
-get_texture_bits_via_copy (CoglTexture    *texture,
-                           int             x,
-                           int             y,
-                           int             width,
-                           int             height,
-                           guint8         *dst_bits,
-                           unsigned int    dst_rowstride,
+static CoglBool
+get_texture_bits_via_copy (CoglTexture *texture,
+                           int x,
+                           int y,
+                           int width,
+                           int height,
+                           uint8_t *dst_bits,
+                           unsigned int dst_rowstride,
                            CoglPixelFormat dst_format)
 {
   unsigned int full_rowstride;
-  guint8 *full_bits;
-  gboolean ret = TRUE;
+  uint8_t *full_bits;
+  CoglBool ret = TRUE;
   int bpp;
   int full_tex_width, full_tex_height;
 
@@ -1015,8 +1075,8 @@ get_texture_bits_via_copy (CoglTexture    *texture,
                                  full_rowstride,
                                  full_bits))
     {
-      guint8 *dst = dst_bits;
-      guint8 *src = full_bits + x * bpp + y * full_rowstride;
+      uint8_t *dst = dst_bits;
+      uint8_t *src = full_bits + x * bpp + y * full_rowstride;
       int i;
 
       for (i = 0; i < height; i++)
@@ -1036,11 +1096,11 @@ get_texture_bits_via_copy (CoglTexture    *texture,
 
 typedef struct
 {
-  int         orig_width;
-  int         orig_height;
+  int orig_width;
+  int orig_height;
   CoglBitmap *target_bmp;
-  guint8     *target_bits;
-  gboolean    success;
+  uint8_t *target_bits;
+  CoglBool success;
 } CoglTextureGetData;
 
 static void
@@ -1065,7 +1125,7 @@ texture_get_cb (CoglTexture *texture,
   int x_in_bitmap = (int) (0.5 + tg_data->orig_width * virtual_coords[0]);
   int y_in_bitmap = (int) (0.5 + tg_data->orig_height * virtual_coords[1]);
 
-  guint8 *dst_bits;
+  uint8_t *dst_bits;
 
   if (!tg_data->success)
     return;
@@ -1110,10 +1170,10 @@ texture_get_cb (CoglTexture *texture,
 }
 
 int
-cogl_texture_get_data (CoglTexture     *texture,
-		       CoglPixelFormat  format,
-		       unsigned int     rowstride,
-		       guint8          *data)
+cogl_texture_get_data (CoglTexture *texture,
+		       CoglPixelFormat format,
+		       unsigned int rowstride,
+		       uint8_t *data)
 {
   int              bpp;
   int              byte_size;
@@ -1149,7 +1209,8 @@ cogl_texture_get_data (CoglTexture     *texture,
     return byte_size;
 
   closest_format =
-    ctx->texture_driver->find_best_gl_get_data_format (format,
+    ctx->texture_driver->find_best_gl_get_data_format (ctx,
+                                                       format,
                                                        &closest_gl_format,
                                                        &closest_gl_type);
 
@@ -1216,7 +1277,7 @@ cogl_texture_get_data (CoglTexture     *texture,
   if (closest_format != format)
     {
       CoglBitmap *new_bmp;
-      gboolean result;
+      CoglBool result;
 
       /* Convert to requested format directly into the user's buffer */
       new_bmp = cogl_bitmap_new_for_data (ctx,
