@@ -96,6 +96,8 @@ _cogl_path_data_unref (CoglPathData *data)
 
       g_array_free (data->path_nodes, TRUE);
 
+      cogl_object_unref (data->context);
+
       g_slice_free (CoglPathData, data);
     }
 }
@@ -122,6 +124,7 @@ _cogl_path_modify (CoglPath *path)
       path->data->fill_attribute_buffer = NULL;
       path->data->stroke_attribute_buffer = NULL;
       path->data->ref_count = 1;
+      cogl_object_ref (path->data->context);
 
       _cogl_path_data_unref (old_data);
     }
@@ -154,7 +157,7 @@ cogl2_path_get_fill_rule (CoglPath *path)
 
 static void
 _cogl_path_add_node (CoglPath *path,
-                     gboolean new_sub_path,
+                     CoglBool new_sub_path,
 		     float x,
 		     float y)
 {
@@ -199,39 +202,37 @@ _cogl_path_add_node (CoglPath *path,
   data->is_rectangle = FALSE;
 }
 
-static void
-_cogl_path_stroke_nodes (CoglPath *path)
+void
+_cogl_path_stroke_nodes (CoglPath *path,
+                         CoglFramebuffer *framebuffer,
+                         CoglPipeline *pipeline)
 {
   CoglPathData *data = path->data;
   CoglPipeline *copy = NULL;
-  CoglPipeline *source;
   unsigned int path_start;
   int path_num = 0;
   CoglPathNode *node;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  if (data->path_nodes->len == 0)
+    return;
 
-  source = cogl_get_source ();
-
-  if (cogl_pipeline_get_n_layers (source) != 0)
+  if (cogl_pipeline_get_n_layers (pipeline) != 0)
     {
-      copy = cogl_pipeline_copy (source);
+      copy = cogl_pipeline_copy (pipeline);
       _cogl_pipeline_prune_to_n_layers (copy, 0);
-      source = copy;
+      pipeline = copy;
     }
 
   _cogl_path_build_stroke_attribute_buffer (path);
 
-  cogl_push_source (source);
-
   for (path_start = 0;
        path_start < data->path_nodes->len;
        path_start += node->path_size)
     {
       node = &g_array_index (data->path_nodes, CoglPathNode, path_start);
 
-      cogl_framebuffer_vdraw_attributes (cogl_get_draw_framebuffer (),
-                                         source,
+      cogl_framebuffer_vdraw_attributes (framebuffer,
+                                         pipeline,
                                          COGL_VERTICES_MODE_LINE_STRIP,
                                          0, node->path_size,
                                          data->stroke_attributes[path_num],
@@ -240,8 +241,6 @@ _cogl_path_stroke_nodes (CoglPath *path)
       path_num++;
     }
 
-  cogl_pop_source ();
-
   if (copy)
     cogl_object_unref (copy);
 }
@@ -272,15 +271,14 @@ _cogl_path_get_bounds (CoglPath *path,
 }
 
 static void
-_cogl_path_fill_nodes_with_clipped_rectangle (CoglPath *path)
+_cogl_path_fill_nodes_with_clipped_rectangle (CoglPath *path,
+                                              CoglFramebuffer *framebuffer,
+                                              CoglPipeline *pipeline)
 {
-  CoglFramebuffer *fb;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  if (!(ctx->private_feature_flags & COGL_PRIVATE_FEATURE_STENCIL_BUFFER))
+  if (!(path->data->context->private_feature_flags &
+        COGL_PRIVATE_FEATURE_STENCIL_BUFFER))
     {
-      static gboolean seen_warning = FALSE;
+      static CoglBool seen_warning = FALSE;
 
       if (!seen_warning)
         {
@@ -291,19 +289,20 @@ _cogl_path_fill_nodes_with_clipped_rectangle (CoglPath *path)
         }
     }
 
-  fb = cogl_get_draw_framebuffer ();
-  cogl_framebuffer_push_path_clip (fb, path);
-  cogl_rectangle (path->data->path_nodes_min.x,
-                  path->data->path_nodes_min.y,
-                  path->data->path_nodes_max.x,
-                  path->data->path_nodes_max.y);
-  cogl_framebuffer_pop_clip (fb);
+  cogl_framebuffer_push_path_clip (framebuffer, path);
+  cogl_framebuffer_draw_rectangle (framebuffer,
+                                   pipeline,
+                                   path->data->path_nodes_min.x,
+                                   path->data->path_nodes_min.y,
+                                   path->data->path_nodes_max.x,
+                                   path->data->path_nodes_max.y);
+  cogl_framebuffer_pop_clip (framebuffer);
 }
 
-static gboolean
+static CoglBool
 validate_layer_cb (CoglPipelineLayer *layer, void *user_data)
 {
-  gboolean *needs_fallback = user_data;
+  CoglBool *needs_fallback = user_data;
   CoglTexture *texture = _cogl_pipeline_layer_get_texture (layer);
 
   /* If any of the layers of the current pipeline contain sliced
@@ -321,30 +320,55 @@ validate_layer_cb (CoglPipelineLayer *layer, void *user_data)
 }
 
 void
-_cogl_path_fill_nodes (CoglPath *path, CoglDrawFlags flags)
+_cogl_path_fill_nodes (CoglPath *path,
+                       CoglFramebuffer *framebuffer,
+                       CoglPipeline *pipeline,
+                       CoglDrawFlags flags)
 {
-  gboolean needs_fallback = FALSE;
-  CoglPipeline *pipeline = cogl_get_source ();
+  if (path->data->path_nodes->len == 0)
+    return;
 
-  _cogl_pipeline_foreach_layer_internal (pipeline,
-                                         validate_layer_cb, &needs_fallback);
-  if (needs_fallback)
+  /* If the path is a simple rectangle then we can divert to using
+     cogl_framebuffer_draw_rectangle which should be faster because it
+     can go through the journal instead of uploading the geometry just
+     for two triangles */
+  if (path->data->is_rectangle && flags == 0)
     {
-      _cogl_path_fill_nodes_with_clipped_rectangle (path);
-      return;
+      float x_1, y_1, x_2, y_2;
+
+      _cogl_path_get_bounds (path, &x_1, &y_1, &x_2, &y_2);
+      cogl_framebuffer_draw_rectangle (framebuffer,
+                                       pipeline,
+                                       x_1, y_1,
+                                       x_2, y_2);
     }
+  else
+    {
+      CoglBool needs_fallback = FALSE;
 
-  _cogl_path_build_fill_attribute_buffer (path);
+      _cogl_pipeline_foreach_layer_internal (pipeline,
+                                             validate_layer_cb,
+                                             &needs_fallback);
+      if (needs_fallback)
+        {
+          _cogl_path_fill_nodes_with_clipped_rectangle (path,
+                                                        framebuffer,
+                                                        pipeline);
+          return;
+        }
 
-  _cogl_framebuffer_draw_indexed_attributes (cogl_get_draw_framebuffer (),
-                                             pipeline,
-                                             COGL_VERTICES_MODE_TRIANGLES,
-                                             0, /* first_vertex */
-                                             path->data->fill_vbo_n_indices,
-                                             path->data->fill_vbo_indices,
-                                             path->data->fill_attributes,
-                                             COGL_PATH_N_ATTRIBUTES,
-                                             flags);
+      _cogl_path_build_fill_attribute_buffer (path);
+
+      _cogl_framebuffer_draw_indexed_attributes (framebuffer,
+                                                 pipeline,
+                                                 COGL_VERTICES_MODE_TRIANGLES,
+                                                 0, /* first_vertex */
+                                                 path->data->fill_vbo_n_indices,
+                                                 path->data->fill_vbo_indices,
+                                                 path->data->fill_attributes,
+                                                 COGL_PATH_N_ATTRIBUTES,
+                                                 flags);
+    }
 }
 
 void
@@ -367,7 +391,10 @@ cogl2_path_fill (CoglPath *path)
       cogl_rectangle (x_1, y_1, x_2, y_2);
     }
   else
-    _cogl_path_fill_nodes (path, 0);
+    _cogl_path_fill_nodes (path,
+                           cogl_get_draw_framebuffer (),
+                           cogl_get_source (),
+                           0);
 }
 
 void
@@ -378,7 +405,9 @@ cogl2_path_stroke (CoglPath *path)
   if (path->data->path_nodes->len == 0)
     return;
 
-  _cogl_path_stroke_nodes (path);
+  _cogl_path_stroke_nodes (path,
+                           cogl_get_draw_framebuffer (),
+                           cogl_get_source ());
 }
 
 void
@@ -502,7 +531,7 @@ cogl2_path_rectangle (CoglPath *path,
                       float x_2,
                       float y_2)
 {
-  gboolean is_rectangle;
+  CoglBool is_rectangle;
 
   /* If the path was previously empty and the rectangle isn't mirrored
      then we'll record that this is a simple rectangle path so that we
@@ -520,7 +549,7 @@ cogl2_path_rectangle (CoglPath *path,
   path->data->is_rectangle = is_rectangle;
 }
 
-gboolean
+CoglBool
 _cogl_path_is_rectangle (CoglPath *path)
 {
   return path->data->is_rectangle;
@@ -873,10 +902,13 @@ cogl2_path_new (void)
   CoglPath *path;
   CoglPathData *data;
 
+  _COGL_GET_CONTEXT (ctx, NULL);
+
   path = g_slice_new (CoglPath);
   data = path->data = g_slice_new (CoglPathData);
 
   data->ref_count = 1;
+  data->context = cogl_object_ref (ctx);
   data->fill_rule = COGL_PATH_FILL_RULE_EVEN_ODD;
   data->path_nodes = g_array_new (FALSE, FALSE, sizeof (CoglPathNode));
   data->last_path = 0;
@@ -1037,7 +1069,7 @@ struct _CoglPathTesselator
      combine callback is called */
   GArray *vertices;
   /* Array of integers for the indices into the vertices array. Each
-     element will either be guint8, guint16 or guint32 depending on
+     element will either be uint8_t, uint16_t or uint32_t depending on
      the number of vertices */
   GArray *indices;
   CoglIndicesType indices_type;
@@ -1079,15 +1111,15 @@ _cogl_path_tesselator_allocate_indices_array (CoglPathTesselator *tess)
   switch (tess->indices_type)
     {
     case COGL_INDICES_TYPE_UNSIGNED_BYTE:
-      tess->indices = g_array_new (FALSE, FALSE, sizeof (guint8));
+      tess->indices = g_array_new (FALSE, FALSE, sizeof (uint8_t));
       break;
 
     case COGL_INDICES_TYPE_UNSIGNED_SHORT:
-      tess->indices = g_array_new (FALSE, FALSE, sizeof (guint16));
+      tess->indices = g_array_new (FALSE, FALSE, sizeof (uint16_t));
       break;
 
     case COGL_INDICES_TYPE_UNSIGNED_INT:
-      tess->indices = g_array_new (FALSE, FALSE, sizeof (guint32));
+      tess->indices = g_array_new (FALSE, FALSE, sizeof (uint32_t));
       break;
     }
 }
@@ -1099,21 +1131,21 @@ _cogl_path_tesselator_add_index (CoglPathTesselator *tess, int vertex_index)
     {
     case COGL_INDICES_TYPE_UNSIGNED_BYTE:
       {
-        guint8 val = vertex_index;
+        uint8_t val = vertex_index;
         g_array_append_val (tess->indices, val);
       }
       break;
 
     case COGL_INDICES_TYPE_UNSIGNED_SHORT:
       {
-        guint16 val = vertex_index;
+        uint16_t val = vertex_index;
         g_array_append_val (tess->indices, val);
       }
       break;
 
     case COGL_INDICES_TYPE_UNSIGNED_INT:
       {
-        guint32 val = vertex_index;
+        uint32_t val = vertex_index;
         g_array_append_val (tess->indices, val);
       }
       break;
@@ -1237,21 +1269,21 @@ _cogl_path_tesselator_combine (double coords[3],
           for (i = 0; i < old_vertices->len; i++)
             _cogl_path_tesselator_add_index (tess,
                                              g_array_index (old_vertices,
-                                                            guint8, i));
+                                                            uint8_t, i));
           break;
 
         case COGL_INDICES_TYPE_UNSIGNED_SHORT:
           for (i = 0; i < old_vertices->len; i++)
             _cogl_path_tesselator_add_index (tess,
                                              g_array_index (old_vertices,
-                                                            guint16, i));
+                                                            uint16_t, i));
           break;
 
         case COGL_INDICES_TYPE_UNSIGNED_INT:
           for (i = 0; i < old_vertices->len; i++)
             _cogl_path_tesselator_add_index (tess,
                                              g_array_index (old_vertices,
-                                                            guint32, i));
+                                                            uint32_t, i));
           break;
         }
 
@@ -1267,8 +1299,6 @@ _cogl_path_build_fill_attribute_buffer (CoglPath *path)
   CoglPathData *data = path->data;
   int i;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   /* If we've already got a vbo then we don't need to do anything */
   if (data->fill_attribute_buffer)
     return;
@@ -1354,7 +1384,7 @@ _cogl_path_build_fill_attribute_buffer (CoglPath *path)
   gluDeleteTess (tess.glu_tess);
 
   data->fill_attribute_buffer =
-    cogl_attribute_buffer_new (ctx,
+    cogl_attribute_buffer_new (data->context,
                                sizeof (CoglPathTesselatorVertex) *
                                tess.vertices->len,
                                tess.vertices->data);
@@ -1375,7 +1405,7 @@ _cogl_path_build_fill_attribute_buffer (CoglPath *path)
                         2, /* n_components */
                         COGL_ATTRIBUTE_TYPE_FLOAT);
 
-  data->fill_vbo_indices = cogl_indices_new (ctx,
+  data->fill_vbo_indices = cogl_indices_new (data->context,
                                              tess.indices_type,
                                              tess.indices->data,
                                              tess.indices->len);
@@ -1394,14 +1424,13 @@ _cogl_path_build_stroke_attribute_buffer (CoglPath *path)
   floatVec2 *buffer_p;
   unsigned int i;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   /* If we've already got a cached vbo then we don't need to do anything */
   if (data->stroke_attribute_buffer)
     return;
 
   data->stroke_attribute_buffer =
-    cogl_attribute_buffer_new (ctx, data->path_nodes->len * sizeof (floatVec2),
+    cogl_attribute_buffer_new (data->context,
+                               data->path_nodes->len * sizeof (floatVec2),
                                NULL);
 
   buffer = COGL_BUFFER (data->stroke_attribute_buffer);
