@@ -38,7 +38,7 @@
 #include "cogl-pipeline.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-primitives.h"
 
 #include <string.h>
@@ -52,15 +52,26 @@
 #define GL_MAX_3D_TEXTURE_SIZE_OES 0x8073
 #endif
 
+/* This extension isn't available for GLES 1.1 so these won't be
+   defined */
+#ifndef GL_UNPACK_ROW_LENGTH
+#define GL_UNPACK_ROW_LENGTH 0x0CF2
+#endif
+#ifndef GL_UNPACK_SKIP_ROWS
+#define GL_UNPACK_SKIP_ROWS 0x0CF3
+#endif
+#ifndef GL_UNPACK_SKIP_PIXELS
+#define GL_UNPACK_SKIP_PIXELS 0x0CF4
+#endif
+
 static void
-_cogl_texture_driver_gen (GLenum   gl_target,
-                          GLsizei  n,
-                          GLuint  *textures)
+_cogl_texture_driver_gen (CoglContext *ctx,
+                          GLenum gl_target,
+                          GLsizei n,
+                          GLuint *textures)
 {
   unsigned int i;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   GE (ctx, glGenTextures (n, textures));
 
   for (i = 0; i < n; i++)
@@ -84,21 +95,54 @@ _cogl_texture_driver_gen (GLenum   gl_target,
 }
 
 static void
-_cogl_texture_driver_prep_gl_for_pixels_upload (int pixels_rowstride,
-                                                int pixels_bpp)
+prep_gl_for_pixels_upload_full (CoglContext *ctx,
+                                int pixels_rowstride,
+                                int pixels_src_x,
+                                int pixels_src_y,
+                                int pixels_bpp)
 {
+  if ((ctx->private_feature_flags & COGL_PRIVATE_FEATURE_UNPACK_SUBIMAGE))
+    {
+      GE( ctx, glPixelStorei (GL_UNPACK_ROW_LENGTH,
+                              pixels_rowstride / pixels_bpp) );
+
+      GE( ctx, glPixelStorei (GL_UNPACK_SKIP_PIXELS, pixels_src_x) );
+      GE( ctx, glPixelStorei (GL_UNPACK_SKIP_ROWS, pixels_src_y) );
+    }
+  else
+    {
+      g_assert (pixels_src_x == 0);
+      g_assert (pixels_src_y == 0);
+    }
+
   _cogl_texture_prep_gl_alignment_for_pixels_upload (pixels_rowstride);
 }
 
 static void
-_cogl_texture_driver_prep_gl_for_pixels_download (int pixels_rowstride,
+_cogl_texture_driver_prep_gl_for_pixels_upload (CoglContext *ctx,
+                                                int pixels_rowstride,
+                                                int pixels_bpp)
+{
+  prep_gl_for_pixels_upload_full (ctx,
+                                  pixels_rowstride,
+                                  0, 0, /* src_x/y */
+                                  pixels_bpp);
+}
+
+static void
+_cogl_texture_driver_prep_gl_for_pixels_download (CoglContext *ctx,
+                                                  int pixels_rowstride,
+                                                  int image_width,
                                                   int pixels_bpp)
 {
-  _cogl_texture_prep_gl_alignment_for_pixels_download (pixels_rowstride);
+  _cogl_texture_prep_gl_alignment_for_pixels_download (pixels_bpp,
+                                                       image_width,
+                                                       pixels_rowstride);
 }
 
 static CoglBitmap *
-prepare_bitmap_alignment_for_upload (CoglBitmap *src_bmp)
+prepare_bitmap_alignment_for_upload (CoglContext *ctx,
+                                     CoglBitmap *src_bmp)
 {
   CoglPixelFormat format = cogl_bitmap_get_format (src_bmp);
   int bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
@@ -106,7 +150,8 @@ prepare_bitmap_alignment_for_upload (CoglBitmap *src_bmp)
   int width = cogl_bitmap_get_width (src_bmp);
   int alignment = 1;
 
-  if (src_rowstride == 0)
+  if ((ctx->private_feature_flags & COGL_PRIVATE_FEATURE_UNPACK_SUBIMAGE) ||
+      src_rowstride == 0)
     return cogl_object_ref (src_bmp);
 
   /* Work out the alignment of the source rowstride */
@@ -124,9 +169,10 @@ prepare_bitmap_alignment_for_upload (CoglBitmap *src_bmp)
 }
 
 static void
-_cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
+_cogl_texture_driver_upload_subregion_to_gl (CoglContext *ctx,
+                                             GLenum       gl_target,
                                              GLuint       gl_handle,
-                                             gboolean     is_foreign,
+                                             CoglBool     is_foreign,
                                              int          src_x,
                                              int          src_y,
                                              int          dst_x,
@@ -137,19 +183,19 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 				             GLuint       source_gl_format,
 				             GLuint       source_gl_type)
 {
-  guint8 *data;
+  uint8_t *data;
   CoglPixelFormat source_format = cogl_bitmap_get_format (source_bmp);
   int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
   CoglBitmap *slice_bmp;
   int rowstride;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* If we are copying a sub region of the source bitmap then we need
-     to copy it because GLES does not support GL_UNPACK_ROW_LENGTH */
-  if (src_x != 0 || src_y != 0 ||
-      width != cogl_bitmap_get_width (source_bmp) ||
-      height != cogl_bitmap_get_height (source_bmp))
+  /* If we have the GL_EXT_unpack_subimage extension then we can
+     upload from subregions directly. Otherwise we may need to copy
+     the bitmap */
+  if (!(ctx->private_feature_flags & COGL_PRIVATE_FEATURE_UNPACK_SUBIMAGE) &&
+      (src_x != 0 || src_y != 0 ||
+       width != cogl_bitmap_get_width (source_bmp) ||
+       height != cogl_bitmap_get_height (source_bmp)))
     {
       slice_bmp =
         _cogl_bitmap_new_with_malloc_buffer (ctx,
@@ -162,12 +208,12 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                    width, height);
     }
   else
-    slice_bmp = prepare_bitmap_alignment_for_upload (source_bmp);
+    slice_bmp = prepare_bitmap_alignment_for_upload (ctx, source_bmp);
 
   rowstride = cogl_bitmap_get_rowstride (slice_bmp);
 
   /* Setup gl alignment to match rowstride and top-left corner */
-  _cogl_texture_driver_prep_gl_for_pixels_upload (rowstride, bpp);
+  prep_gl_for_pixels_upload_full (ctx, rowstride, src_x, src_y, bpp);
 
   data = _cogl_bitmap_bind (slice_bmp, COGL_BUFFER_ACCESS_READ, 0);
 
@@ -186,9 +232,10 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 }
 
 static void
-_cogl_texture_driver_upload_to_gl (GLenum       gl_target,
+_cogl_texture_driver_upload_to_gl (CoglContext *ctx,
+                                   GLenum       gl_target,
                                    GLuint       gl_handle,
-                                   gboolean     is_foreign,
+                                   CoglBool     is_foreign,
                                    CoglBitmap  *source_bmp,
                                    GLint        internal_gl_format,
                                    GLuint       source_gl_format,
@@ -200,15 +247,13 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
   int bmp_width = cogl_bitmap_get_width (source_bmp);
   int bmp_height = cogl_bitmap_get_height (source_bmp);
   CoglBitmap *bmp;
-  guint8 *data;
+  uint8_t *data;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  bmp = prepare_bitmap_alignment_for_upload (source_bmp);
+  bmp = prepare_bitmap_alignment_for_upload (ctx, source_bmp);
   rowstride = cogl_bitmap_get_rowstride (bmp);
 
   /* Setup gl alignment to match rowstride and top-left corner */
-  _cogl_texture_driver_prep_gl_for_pixels_upload (rowstride, bpp);
+  _cogl_texture_driver_prep_gl_for_pixels_upload (ctx, rowstride, bpp);
 
   _cogl_bind_gl_texture_transient (gl_target, gl_handle, is_foreign);
 
@@ -228,9 +273,10 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
 }
 
 static void
-_cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
+_cogl_texture_driver_upload_to_gl_3d (CoglContext *ctx,
+                                      GLenum       gl_target,
                                       GLuint       gl_handle,
-                                      gboolean     is_foreign,
+                                      CoglBool     is_foreign,
                                       GLint        height,
                                       GLint        depth,
                                       CoglBitmap  *source_bmp,
@@ -243,9 +289,7 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
   int rowstride = cogl_bitmap_get_rowstride (source_bmp);
   int bmp_width = cogl_bitmap_get_width (source_bmp);
   int bmp_height = cogl_bitmap_get_height (source_bmp);
-  guint8 *data;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  uint8_t *data;
 
   _cogl_bind_gl_texture_transient (gl_target, gl_handle, is_foreign);
 
@@ -260,7 +304,7 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
       CoglPixelFormat source_bmp_format = cogl_bitmap_get_format (source_bmp);
       int i;
 
-      _cogl_texture_driver_prep_gl_for_pixels_upload (bmp_width * bpp, bpp);
+      _cogl_texture_driver_prep_gl_for_pixels_upload (ctx, bmp_width * bpp, bpp);
 
       /* Initialize the texture with empty data and then upload each
          image with a sub-region update */
@@ -314,7 +358,7 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
     {
       data = _cogl_bitmap_bind (source_bmp, COGL_BUFFER_ACCESS_READ, 0);
 
-      _cogl_texture_driver_prep_gl_for_pixels_upload (rowstride, bpp);
+      _cogl_texture_driver_prep_gl_for_pixels_upload (ctx, rowstride, bpp);
 
       GE( ctx, glTexImage3D (gl_target,
                              0, /* level */
@@ -334,27 +378,27 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
 /* NB: GLES doesn't support glGetTexImage2D, so cogl-texture will instead
  * fallback to a generic render + readpixels approach to downloading
  * texture data. (See _cogl_texture_draw_and_read() ) */
-static gboolean
-_cogl_texture_driver_gl_get_tex_image (GLenum  gl_target,
-                                       GLenum  dest_gl_format,
-                                       GLenum  dest_gl_type,
-                                       guint8 *dest)
+static CoglBool
+_cogl_texture_driver_gl_get_tex_image (CoglContext *ctx,
+                                       GLenum gl_target,
+                                       GLenum dest_gl_format,
+                                       GLenum dest_gl_type,
+                                       uint8_t *dest)
 {
   return FALSE;
 }
 
-static gboolean
-_cogl_texture_driver_size_supported_3d (GLenum gl_target,
+static CoglBool
+_cogl_texture_driver_size_supported_3d (CoglContext *ctx,
+                                        GLenum gl_target,
                                         GLenum gl_format,
                                         GLenum gl_type,
-                                        int    width,
-                                        int    height,
-                                        int    depth)
+                                        int width,
+                                        int height,
+                                        int depth)
 {
   GLint max_size;
 
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
   /* GLES doesn't support a proxy texture target so let's at least
      check whether the size is greater than
      GL_MAX_3D_TEXTURE_SIZE_OES */
@@ -363,17 +407,17 @@ _cogl_texture_driver_size_supported_3d (GLenum gl_target,
   return width <= max_size && height <= max_size && depth <= max_size;
 }
 
-static gboolean
-_cogl_texture_driver_size_supported (GLenum gl_target,
+static CoglBool
+_cogl_texture_driver_size_supported (CoglContext *ctx,
+                                     GLenum gl_target,
+                                     GLenum gl_intformat,
                                      GLenum gl_format,
                                      GLenum gl_type,
-                                     int    width,
-                                     int    height)
+                                     int width,
+                                     int height)
 {
   GLint max_size;
 
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
   /* GLES doesn't support a proxy texture target so let's at least
      check whether the size is greater than GL_MAX_TEXTURE_SIZE */
   GE( ctx, glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_size) );
@@ -382,123 +426,17 @@ _cogl_texture_driver_size_supported (GLenum gl_target,
 }
 
 static void
-_cogl_texture_driver_try_setting_gl_border_color (
-                                              GLuint   gl_target,
-                                              const GLfloat *transparent_color)
+_cogl_texture_driver_try_setting_gl_border_color
+                                            (CoglContext *ctx,
+                                             GLuint gl_target,
+                                             const GLfloat *transparent_color)
 {
   /* FAIL! */
 }
 
-static gboolean
-_cogl_texture_driver_pixel_format_from_gl_internal (GLenum gl_int_format,
-                                                    CoglPixelFormat *out_format)
-{
-  return TRUE;
-}
-
-static CoglPixelFormat
-_cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
-                                         GLenum          *out_glintformat,
-                                         GLenum          *out_glformat,
-                                         GLenum          *out_gltype)
-{
-  CoglPixelFormat required_format;
-  GLenum glintformat;
-  GLenum glformat = 0;
-  GLenum gltype;
-
-  required_format = format;
-
-  /* Find GL equivalents */
-  switch (format)
-    {
-    case COGL_PIXEL_FORMAT_A_8:
-      glintformat = GL_ALPHA;
-      glformat = GL_ALPHA;
-      gltype = GL_UNSIGNED_BYTE;
-      break;
-    case COGL_PIXEL_FORMAT_G_8:
-      glintformat = GL_LUMINANCE;
-      glformat = GL_LUMINANCE;
-      gltype = GL_UNSIGNED_BYTE;
-      break;
-
-      /* Just one 24-bit ordering supported */
-    case COGL_PIXEL_FORMAT_RGB_888:
-    case COGL_PIXEL_FORMAT_BGR_888:
-      glintformat = GL_RGB;
-      glformat = GL_RGB;
-      gltype = GL_UNSIGNED_BYTE;
-      required_format = COGL_PIXEL_FORMAT_RGB_888;
-      break;
-
-      /* Just one 32-bit ordering supported */
-    case COGL_PIXEL_FORMAT_RGBA_8888:
-    case COGL_PIXEL_FORMAT_RGBA_8888_PRE:
-    case COGL_PIXEL_FORMAT_BGRA_8888:
-    case COGL_PIXEL_FORMAT_BGRA_8888_PRE:
-    case COGL_PIXEL_FORMAT_ARGB_8888:
-    case COGL_PIXEL_FORMAT_ARGB_8888_PRE:
-    case COGL_PIXEL_FORMAT_ABGR_8888:
-    case COGL_PIXEL_FORMAT_ABGR_8888_PRE:
-    case COGL_PIXEL_FORMAT_RGBA_1010102:
-    case COGL_PIXEL_FORMAT_RGBA_1010102_PRE:
-    case COGL_PIXEL_FORMAT_BGRA_1010102:
-    case COGL_PIXEL_FORMAT_BGRA_1010102_PRE:
-    case COGL_PIXEL_FORMAT_ABGR_2101010:
-    case COGL_PIXEL_FORMAT_ABGR_2101010_PRE:
-    case COGL_PIXEL_FORMAT_ARGB_2101010:
-    case COGL_PIXEL_FORMAT_ARGB_2101010_PRE:
-      glintformat = GL_RGBA;
-      glformat = GL_RGBA;
-      gltype = GL_UNSIGNED_BYTE;
-      required_format = COGL_PIXEL_FORMAT_RGBA_8888;
-      required_format |= (format & COGL_PREMULT_BIT);
-      break;
-
-      /* The following three types of channel ordering
-       * are always defined using system word byte
-       * ordering (even according to GLES spec) */
-    case COGL_PIXEL_FORMAT_RGB_565:
-      glintformat = GL_RGB;
-      glformat = GL_RGB;
-      gltype = GL_UNSIGNED_SHORT_5_6_5;
-      break;
-    case COGL_PIXEL_FORMAT_RGBA_4444:
-    case COGL_PIXEL_FORMAT_RGBA_4444_PRE:
-      glintformat = GL_RGBA;
-      glformat = GL_RGBA;
-      gltype = GL_UNSIGNED_SHORT_4_4_4_4;
-      break;
-    case COGL_PIXEL_FORMAT_RGBA_5551:
-    case COGL_PIXEL_FORMAT_RGBA_5551_PRE:
-      glintformat = GL_RGBA;
-      glformat = GL_RGBA;
-      gltype = GL_UNSIGNED_SHORT_5_5_5_1;
-      break;
-
-    case COGL_PIXEL_FORMAT_ANY:
-    case COGL_PIXEL_FORMAT_YUV:
-      g_assert_not_reached ();
-      break;
-    }
-
-  /* All of the pixel formats are handled above so if this hits then
-     we've been given an invalid pixel format */
-  g_assert (glformat != 0);
-
-  if (out_glintformat != NULL)
-    *out_glintformat = glintformat;
-  if (out_glformat != NULL)
-    *out_glformat = glformat;
-  if (out_gltype != NULL)
-    *out_gltype = gltype;
-
-  return required_format;
-}
-
-static gboolean
-_cogl_texture_driver_allows_foreign_gl_target (GLenum gl_target)
+static CoglBool
+_cogl_texture_driver_allows_foreign_gl_target (CoglContext *ctx,
+                                               GLenum gl_target)
 {
   /* Allow 2-dimensional textures only */
   if (gl_target != GL_TEXTURE_2D)
@@ -507,19 +445,19 @@ _cogl_texture_driver_allows_foreign_gl_target (GLenum gl_target)
 }
 
 static void
-_cogl_texture_driver_gl_generate_mipmaps (GLenum gl_target)
+_cogl_texture_driver_gl_generate_mipmaps (CoglContext *ctx,
+                                          GLenum gl_target)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   if (ctx->driver == COGL_DRIVER_GLES2)
     GE( ctx, glGenerateMipmap (gl_target) );
 }
 
 static CoglPixelFormat
-_cogl_texture_driver_find_best_gl_get_data_format (
-                                             CoglPixelFormat  format,
-                                             GLenum          *closest_gl_format,
-                                             GLenum          *closest_gl_type)
+_cogl_texture_driver_find_best_gl_get_data_format
+                                            (CoglContext *context,
+                                             CoglPixelFormat format,
+                                             GLenum *closest_gl_format,
+                                             GLenum *closest_gl_type)
 {
   /* Find closest format that's supported by GL
      (Can't use _cogl_pixel_format_to_gl since available formats
@@ -542,8 +480,6 @@ _cogl_texture_driver_gles =
     _cogl_texture_driver_size_supported,
     _cogl_texture_driver_size_supported_3d,
     _cogl_texture_driver_try_setting_gl_border_color,
-    _cogl_texture_driver_pixel_format_from_gl_internal,
-    _cogl_texture_driver_pixel_format_to_gl,
     _cogl_texture_driver_allows_foreign_gl_target,
     _cogl_texture_driver_gl_generate_mipmaps,
     _cogl_texture_driver_find_best_gl_get_data_format
